@@ -1,4 +1,4 @@
-/* $Id: data.h,v 1.4 2000/02/06 22:12:32 tausq Exp $ */
+/* $Id: data.h,v 1.7 2001/05/23 17:39:54 joeyh Exp $ */
 #ifndef _DATA_H
 #define _DATA_H
 
@@ -13,7 +13,6 @@ typedef enum {
 
 struct package_t {
   char *name;
-  char *prettyname;
   char *shortdesc;
   char *longdesc;
   int dependscount;
@@ -22,22 +21,41 @@ struct package_t {
   char **recommends;
   int suggestscount;
   char **suggests;
+  char *section;
   priority_t priority;
   int selected;
+  int pseudopackage;
 };
 
 struct packages_t {
   int count;
-  int maxnamelen;
   void *packages;
 };
 
-/* Reads in a list of package and package descriptions */
-void packages_readlist(struct packages_t *taskpackages, struct packages_t *packages);
+struct task_t {
+  char *name;
+  struct package_t *task_pkg;
+  char **packages;
+  int packagescount;
+  int packagesmax;
+  int selected;
+};
+
+struct tasks_t {
+  int count;
+  void *tasks;
+};
+
+/* Reads in a list of tasks from task description file */
+void taskfile_read(char *fn, struct tasks_t *tasks, struct packages_t *pkgs, unsigned char showempties);
+/* Reads in a list of package and package descriptions from available file */
+void packages_readlist(struct tasks_t *tasks, struct packages_t *packages);
 /* free memory allocated to store packages */
-void packages_free(struct packages_t *taskpackages, struct packages_t *packages);
+void packages_free(struct tasks_t *tasks, struct packages_t *packages);
 
 struct package_t *packages_find(const struct packages_t *packages, const char *name);
+struct task_t *tasks_find(const struct tasks_t *tasks, const char *name);
 struct package_t **packages_enumerate(const struct packages_t *packages);
+struct task_t **tasks_enumerate(const struct tasks_t *tasks);
 
 #endif
