@@ -1,4 +1,4 @@
-/* $Id: macros.h,v 1.2 1999/11/23 05:39:27 tausq Exp $ */
+/* $Id: macros.h,v 1.4 2001/11/22 17:53:48 tausq Exp $ */
 #ifndef _MACROS_H
 #define _MACROS_H
 
@@ -22,6 +22,7 @@
 #define ABORT abort()
 #define STRDUP(s) safe_strdup(s)
 #define MALLOC(sz) safe_malloc(sz)
+#define REALLOC(x,sz) safe_realloc(x,sz)
 #define FREE(p) safe_free((void **)&p);
 #else
 #define DPRINTF(fmt, arg...)
@@ -30,6 +31,7 @@
 #define ABORT exit(255)
 #define STRDUP(s) (s ? strdup(s) : NULL)
 #define MALLOC(sz) malloc(sz)
+#define REALLOC(x,sz) realloc(x,sz)
 #define FREE(p) if (p) free(p)
 #endif
   
@@ -37,7 +39,7 @@
 #define DIE(fmt, arg...) \
   do { \
     if (ui_running()) ui_shutdown(); \
-    fprintf(stderr, "Fatal error encountered at %s:%d\r\n\t", __FILE__, __LINE__); \
+    fprintf(stderr, _("Fatal error encountered at %s:%d\r\n\t"), __FILE__, __LINE__); \
     fprintf(stderr, fmt, ##arg); \
     fprintf(stderr, "\r\n"); \
     ABORT; \
@@ -46,12 +48,14 @@
 #define PERROR(ctx) \
   do { \
     if (ui_running()) ui_shutdown(); \
-    fprintf(stderr, "I/O error at %s:%d\r\n\t", __FILE__, __LINE__); \
+    fprintf(stderr, _("I/O error at %s:%d\r\n\t"), __FILE__, __LINE__); \
     fprintf(stderr, "%s: %s\r\n", ctx, strerror(errno)); \
     ABORT; \
   } while (0);
   
 #define NEW(S) (S *)MALLOC(sizeof(S))
 #define _(s)   gettext(s)
+#define gettext_noop(s)  (s)
+#define N_(s) gettext_noop (s)
   
 #endif
